@@ -17,13 +17,10 @@ import numpy as np
 
 from tespy.components.component import component_registry
 from tespy.components.heat_exchangers.base import HeatExchanger
-from tespy.tools.data_containers import ComponentCharacteristics as dc_cc
-from tespy.tools.data_containers import ComponentProperties as dc_cp
-from tespy.tools.data_containers import GroupedComponentCharacteristics as dc_gcc
+from tespy.tools import logger
 from tespy.tools.data_containers import SimpleDataContainer as dc_simple
-from tespy.tools.document_models import generate_latex_eq
-from tespy.tools.fluid_properties import dh_mix_dpQ
 from tespy.tools.fluid_properties import h_mix_pQ
+from tespy.tools.fluid_properties import single_fluid
 
 
 @component_registry
@@ -36,8 +33,8 @@ class Condenser(HeatExchanger):
 
     **Mandatory Equations**
 
-    - :py:meth:`tespy.components.component.Component.fluid_func`
-    - :py:meth:`tespy.components.component.Component.mass_flow_func`
+    - fluid: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
+    - mass flow: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
     - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.energy_balance_func`
     - condensate outlet state, function can be disabled by specifying
       :code:`set_attr(subcooling=True)`
@@ -45,15 +42,24 @@ class Condenser(HeatExchanger):
 
     **Optional Equations**
 
+    The :code:`Condenser` class uses an individual definition for the
+    calculation of the logarithmic temperature difference
+
     - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.energy_balance_hot_func`
-    - :py:meth:`tespy.components.heat_exchangers.condenser.Condenser.kA_func`
-    - :py:meth:`tespy.components.heat_exchangers.condenser.Condenser.kA_char_func`
-    - :py:meth:`tespy.components.heat_exchangers.condenser.Condenser.ttd_u_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.kA_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.kA_char_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.ttd_u_func`
     - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.ttd_l_func`
-    - hot side :py:meth:`tespy.components.component.Component.pr_func`
-    - cold side :py:meth:`tespy.components.component.Component.pr_func`
-    - hot side :py:meth:`tespy.components.component.Component.zeta_func`
-    - cold side :py:meth:`tespy.components.component.Component.zeta_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.ttd_min_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.eff_cold_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.eff_hot_func`
+    - :py:meth:`tespy.components.heat_exchangers.base.HeatExchanger.eff_max_func`
+
+    For hot and cold side individually:
+
+    - :py:meth:`tespy.components.component.Component.pr_structure_matrix`
+    - :py:meth:`tespy.components.component.Component.dp_structure_matrix`
+    - :py:meth:`tespy.components.component.Component.zeta_func`
 
     Inlets/Outlets
 
@@ -107,6 +113,14 @@ class Condenser(HeatExchanger):
     pr2 : float, dict, :code:`"var"`
         Outlet to inlet pressure ratio at cold side, :math:`pr/1`.
 
+    dp1 : float, dict, :code:`"var"`
+        Inlet to outlet pressure delta at hot side, unit is the network's
+        pressure unit!.
+
+    dp2 : float, dict, :code:`"var"`
+        Inlet to outlet pressure delta at cold side, unit is the network's
+        pressure unit!.
+
     zeta1 : float, dict, :code:`"var"`
         Geometry independent friction coefficient at hot side,
         :math:`\frac{\zeta}{D^4}/\frac{1}{\text{m}^4}`.
@@ -122,6 +136,19 @@ class Condenser(HeatExchanger):
         Upper terminal temperature difference (referring to saturation
         temprature of condensing fluid) :math:`ttd_\mathrm{u}/\text{K}`.
 
+    ttd_min : float, dict
+        Minumum terminal temperature difference :math:`ttd_\mathrm{min}/\text{K}`.
+
+    eff_cold : float, dict
+        Cold side heat exchanger effectiveness :math:`eff_\text{cold}/\text{1}`.
+
+    eff_hot : float, dict
+        Hot side heat exchanger effectiveness :math:`eff_\text{hot}/\text{1}`.
+
+    eff_max : float, dict
+        Max value of hot and cold side heat exchanger effectiveness values
+        :math:`eff_\text{max}/\text{1}`.
+
     kA : float, dict
         Area independent heat transfer coefficient,
         :math:`kA/\frac{\text{W}}{\text{K}}`.
@@ -159,16 +186,16 @@ class Condenser(HeatExchanger):
     >>> from tespy.connections import Connection
     >>> from tespy.networks import Network
     >>> from tespy.tools.fluid_properties import T_sat_p
-    >>> import shutil
-    >>> nw = Network(T_unit='C', p_unit='bar', h_unit='kJ / kg',
-    ... m_range=[0.01, 1000], iterinfo=False)
+    >>> import os
+    >>> nw = Network(m_range=[0.01, 1000], iterinfo=False)
+    >>> nw.units.set_defaults(**{
+    ...     "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+    ... })
     >>> amb_in = Source('ambient air inlet')
     >>> amb_out = Sink('air outlet')
     >>> waste_steam = Source('waste steam')
     >>> c = Sink('condensate sink')
     >>> cond = Condenser('condenser')
-    >>> cond.component()
-    'condenser'
     >>> amb_he = Connection(amb_in, 'out1', cond, 'in2')
     >>> he_amb = Connection(cond, 'out2', amb_out, 'in1')
     >>> ws_he = Connection(waste_steam, 'out1', cond, 'in1')
@@ -185,7 +212,7 @@ class Condenser(HeatExchanger):
     >>> amb_he.set_attr(fluid={'air': 1}, T=20, offdesign=['v'])
     >>> he_amb.set_attr(p=1, T=40, design=['T'])
     >>> nw.solve('design')
-    >>> nw.save('tmp')
+    >>> nw.save('tmp.json')
     >>> round(amb_he.v.val, 2)
     103.17
     >>> round(ws_he.T.val - he_amb.T.val, 1)
@@ -194,7 +221,7 @@ class Condenser(HeatExchanger):
     15.0
     >>> ws_he.set_attr(m=0.7)
     >>> amb_he.set_attr(T=30)
-    >>> nw.solve('offdesign', design_path='tmp')
+    >>> nw.solve('offdesign', design_path='tmp.json')
     >>> round(ws_he.T.val - he_amb.T.val, 1)
     62.5
     >>> round(ws_he.calc_T_sat() - 273.15 - he_amb.T.val, 1)
@@ -204,69 +231,31 @@ class Condenser(HeatExchanger):
     temperature is specified to 5 K.
 
     >>> cond.set_attr(subcooling=True)
-    >>> he_c.set_attr(Td_bp=-5)
-    >>> nw.solve('offdesign', design_path='tmp')
+    >>> he_c.set_attr(td_bubble=5)
+    >>> nw.solve('offdesign', design_path='tmp.json')
     >>> round(ws_he.T.val - he_amb.T.val, 1)
     62.5
     >>> round(ws_he.calc_T_sat() - 273.15 - he_amb.T.val, 1)
     13.4
-    >>> shutil.rmtree('./tmp', ignore_errors=True)
+    >>> os.remove('tmp.json')
     """
 
-    @staticmethod
-    def component():
-        return 'condenser'
-
     def get_parameters(self):
-        return {
-            'Q': dc_cp(
-                max_val=0, func=self.energy_balance_hot_func, num_eq=1,
-                deriv=self.energy_balance_hot_deriv,
-                latex=self.energy_balance_hot_func_doc),
-            'kA': dc_cp(
-                min_val=0, num_eq=1, func=self.kA_func, latex=self.kA_func_doc,
-                deriv=self.kA_deriv),
-            'td_log': dc_cp(min_val=0, is_result=True),
-            'ttd_u': dc_cp(
-                min_val=0, num_eq=1, func=self.ttd_u_func,
-                deriv=self.ttd_u_deriv, latex=self.ttd_u_func_doc),
-            'ttd_l': dc_cp(
-                min_val=0, num_eq=1, func=self.ttd_l_func,
-                deriv=self.ttd_l_deriv, latex=self.ttd_l_func_doc),
-            'pr1': dc_cp(
-                min_val=1e-4, max_val=1, num_eq=1, deriv=self.pr_deriv,
-                latex=self.pr_func_doc,
-                func=self.pr_func, func_params={'pr': 'pr1'}),
-            'pr2': dc_cp(
-                min_val=1e-4, max_val=1, num_eq=1, latex=self.pr_func_doc,
-                deriv=self.pr_deriv, func=self.pr_func,
-                func_params={'pr': 'pr2', 'inconn': 1, 'outconn': 1}),
-            'zeta1': dc_cp(
-                min_val=0, max_val=1e15, num_eq=1, latex=self.zeta_func_doc,
-                deriv=self.zeta_deriv, func=self.zeta_func,
-                func_params={'zeta': 'zeta1'}),
-            'zeta2': dc_cp(
-                min_val=0, max_val=1e15, num_eq=1, latex=self.zeta_func_doc,
-                deriv=self.zeta_deriv, func=self.zeta_func,
-                func_params={'zeta': 'zeta2', 'inconn': 1, 'outconn': 1}),
-            'kA_char': dc_gcc(
-                elements=['kA_char1', 'kA_char2'],
-                num_eq=1, latex=self.kA_char_func_doc, func=self.kA_char_func,
-                deriv=self.kA_char_deriv),
-            'kA_char1': dc_cc(param='m'),
-            'kA_char2': dc_cc(
-                param='m', char_params={
-                    'type': 'rel', 'inconn': 1, 'outconn': 1}),
+        params = super().get_parameters()
+        params.update({
             'subcooling': dc_simple(
-                val=False, num_eq=1, latex=self.subcooling_func_doc,
-                deriv=self.subcooling_deriv, func=self.subcooling_func)
-        }
+                _val=False, num_eq_sets=1,
+                func=self.subcooling_func,
+                dependents=self.subcooling_dependents,
+            )
+        })
+        return params
 
-    def preprocess(self, num_nw_vars):
+    def _preprocess(self, row_idx):
 
         # if subcooling is True, outlet state method must not be calculated
         self.subcooling.is_set = not self.subcooling.val
-        super().preprocess(num_nw_vars)
+        super()._preprocess(row_idx)
 
     def subcooling_func(self):
         r"""
@@ -288,40 +277,11 @@ class Condenser(HeatExchanger):
         o = self.outl[0]
         return o.h.val_SI - h_mix_pQ(o.p.val_SI, 0, o.fluid_data)
 
-    def subcooling_func_doc(self, label):
-        r"""
-        Equation for hot side outlet state.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = r'0=h_\mathrm{out,1} -h\left(p_\mathrm{out,1}, x=0 \right)'
-        return generate_latex_eq(self, latex, label)
-
-    def subcooling_deriv(self, increment_filter, k):
-        """
-        Calculate partial derivates of subcooling function.
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        o = self.outl[0]
-        if self.is_variable(o.p):
-            self.jacobian[k, o.p.J_col] = -dh_mix_dpQ(o.p.val_SI, 0, o.fluid_data)
-        if self.is_variable(o.h):
-            self.jacobian[k, o.h.J_col] = 1
+    def subcooling_dependents(self):
+        return [
+            self.outl[0].p,
+            self.outl[0].h
+        ]
 
     def calculate_td_log(self):
 
@@ -346,38 +306,13 @@ class Condenser(HeatExchanger):
 
         ttd_u = T_i1 - T_o2
         ttd_l = T_o1 - T_i2
-        if ttd_u == ttd_l:
+        if round(ttd_u, 6) == round(ttd_l, 6):
             td_log = ttd_l
         else:
             td_log = (ttd_l - ttd_u) / math.log((ttd_l) / (ttd_u))
 
         return td_log
 
-    def kA_func_doc(self, label):
-        r"""
-        Calculate heat transfer from heat transfer coefficient.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0 = \dot{m}_\mathrm{in,1} \cdot \left( h_\mathrm{out,1} - '
-            r'h_\mathrm{in,1}\right)+ kA \cdot \frac{T_\mathrm{out,1} - '
-            r'T_\mathrm{in,2} -T_\mathrm{sat}\left( p_\mathrm{in,1}\right)'
-            r'+ T_\mathrm{out,2}}'
-            r'{\ln{\frac{T_\mathrm{out,1} - T_\mathrm{in,2}}'
-            r'{T_\mathrm{sat}\left( p_\mathrm{in,1}\right) -'
-            r'T_\mathrm{out,2}}}}'
-        )
-        return generate_latex_eq(self, latex, label)
-
     def kA_char_func(self):
         r"""
         Calculate heat transfer from heat transfer coefficient characteristic.
@@ -401,40 +336,10 @@ class Condenser(HeatExchanger):
         Note
         ----
         For standard functions f\ :subscript:`1` \ and f\ :subscript:`2` \ see
-        module :py:mod:`tespy.data`.
+        module :ref:`tespy.data <tespy_data_label>`.
         """
         return super().kA_char_func()
 
-    def kA_char_func_doc(self, label):
-        r"""
-        Calculate heat transfer from heat transfer coefficient characteristic.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'\begin{split}' + '\n'
-            r'0 = & \dot{m}_\mathrm{in,1} \cdot \left( h_\mathrm{out,1} - '
-            r'h_\mathrm{in,1}\right)\\' + '\n'
-            r'&+kA_\mathrm{design} \cdot '
-            r'f_\mathrm{kA} \cdot \frac{T_\mathrm{out,1} - T_\mathrm{in,2}'
-            r' - T_\mathrm{sat}\left( p_\mathrm{in,1}\right) +'
-            r'T_\mathrm{out,2}}{\ln{\frac{T_\mathrm{out,1}-'
-            r'T_\mathrm{in,2}}{T_\mathrm{sat}\left( p_\mathrm{in,1}\right)'
-            r'- T_\mathrm{out,2}}}}\\' + '\n'
-            r'f_\mathrm{kA}=&\frac{2}{\frac{1}{f\left(X_1\right)}+'
-            r'\frac{1}{f\left(X_2\right)}}\\' + '\n'
-            r'\end{split}'
-        )
-        return generate_latex_eq(self, latex, label)
-
     def ttd_u_func(self):
         r"""
         Equation for upper terminal temperature difference.
@@ -457,54 +362,107 @@ class Condenser(HeatExchanger):
         o = self.outl[1]
         T_i1 = i.calc_T_sat()
         T_o2 = o.calc_T()
-        return self.ttd_u.val - T_i1 + T_o2
+        return self.ttd_u.val_SI - T_i1 + T_o2
 
-    def ttd_u_func_doc(self, label):
-        r"""
-        Equation for upper terminal temperature difference.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0=ttd_\mathrm{u}-T_\mathrm{sat}\left(p_\mathrm{in,1}\right)'
-            r' + T_\mathrm{out,2}')
-        return generate_latex_eq(self, latex, label)
+    def ttd_u_dependents(self):
+        return [
+            self.inl[0].p,
+            self.outl[1].p,
+            self.outl[1].h,
+        ]
 
     def calc_parameters(self):
         r"""Postprocessing parameter calculation."""
-        # component parameters
-        i1 = self.inl[0]
-        i2 = self.inl[1]
-        o1 = self.outl[0]
-        o2 = self.outl[1]
-        self.Q.val = i1.m.val_SI * (o1.h.val_SI - i1.h.val_SI)
-        self.ttd_u.val = i1.calc_T_sat() - o2.T.val_SI
-        self.ttd_l.val = o1.T.val_SI - i2.T.val_SI
+        self.Q.val_SI = self.inl[0].m.val_SI * (
+            self.outl[0].h.val_SI - self.inl[0].h.val_SI
+        )
+        self.ttd_u.val_SI = self.inl[0].calc_T_sat() - self.outl[1].T.val_SI
+        self.ttd_l.val_SI = self.outl[0].T.val_SI - self.inl[1].T.val_SI
+        self.ttd_min.val_SI = min(self.ttd_u.val_SI, self.ttd_l.val_SI)
 
         # pr and zeta
-        for num, (i, o) in enumerate(zip(self.inl, self.outl)):
-            self.get_attr(f"pr{num + 1}").val = o.p.val_SI / i.p.val_SI
-            self.get_attr(f"zeta{num + 1}").val = (
-                (i.p.val_SI - o.p.val_SI) * math.pi ** 2
-                / (4 * i.m.val_SI ** 2 * (i.vol.val_SI + o.vol.val_SI))
+        for i in range(2):
+            self.get_attr(f'pr{i + 1}').val_SI = (
+                self.outl[i].p.val_SI / self.inl[i].p.val_SI
+            )
+            self.get_attr(f'zeta{i + 1}').val_SI = self.calc_zeta(
+                self.inl[i], self.outl[i]
+            )
+            self.get_attr(f'dp{i + 1}').val_SI = (
+                self.inl[i].p.val_SI - self.outl[i].p.val_SI
             )
 
         # kA and logarithmic temperature difference
-        if self.ttd_u.val < 0 or self.ttd_l.val < 0:
-            self.td_log.val = np.nan
-        elif self.ttd_l.val == self.ttd_u.val:
-            self.td_log.val = self.ttd_l.val
+        if self.ttd_u.val_SI < 0 or self.ttd_l.val_SI < 0:
+            self.td_log.val_SI = np.nan
+        elif round(self.ttd_l.val_SI, 6) == round(self.ttd_u.val_SI, 6):
+            self.td_log.val_SI = self.ttd_l.val_SI
+        elif round(self.ttd_l.val_SI, 6) == 0 or round(self.ttd_u.val_SI, 6) == 0:
+            self.td_log.val_SI = np.nan
         else:
-            self.td_log.val = (
-                (self.ttd_l.val - self.ttd_u.val)
-                / math.log(self.ttd_l.val / self.ttd_u.val)
+            self.td_log.val_SI = (
+                (self.ttd_l.val_SI - self.ttd_u.val_SI)
+                / math.log(self.ttd_l.val_SI / self.ttd_u.val_SI)
             )
-        self.kA.val = -self.Q.val / self.td_log.val
+
+        self.kA.val_SI = -self.Q.val_SI / self.td_log.val_SI
+
+        # heat exchanger efficiencies
+        try:
+            self.eff_hot.val_SI = (
+                (self.outl[0].h.val_SI - self.inl[0].h.val_SI)
+                / self.calc_dh_max_hot()
+            )
+        except ValueError:
+            self.eff_hot.val_SI = np.nan
+            msg = (
+                f"Cannot calculate {self.label} hot side effectiveness "
+                "because cold side inlet temperature is out of bounds for hot "
+                "side fluid."
+            )
+            logger.warning(msg)
+        try:
+            self.eff_cold.val_SI = (
+                (self.outl[1].h.val_SI - self.inl[1].h.val_SI)
+                / self.calc_dh_max_cold()
+            )
+        except ValueError:
+            self.eff_cold.val_SI = np.nan
+            msg = (
+                f"Cannot calculate {self.label} cold side effectiveness "
+                "because hot side inlet temperature is out of bounds for cold "
+                "side fluid."
+            )
+            logger.warning(msg)
+        self.eff_max.val_SI = max(self.eff_hot.val_SI, self.eff_cold.val_SI)
+
+    def convergence_check(self):
+        o = self.outl[0]
+        if o.p.is_var:
+            fluid = single_fluid(o.fluid_data)
+            p_crit = o.fluid.wrapper[fluid]._p_crit
+            if o.p.val_SI > p_crit:
+                o.p.set_reference_val_SI(p_crit * 0.9)
+
+    def initialise_source(self, c, key):
+        r"""
+        Return a starting value for pressure and enthalpy at outlet.
+
+        Parameters
+        ----------
+        c : tespy.connections.connection.Connection
+            Connection to perform initialisation on.
+
+        key : str
+            Fluid property to retrieve.
+
+        Returns
+        -------
+        val : float
+            Starting value for pressure/enthalpy in SI units.
+        """
+        if c.source_id == 'out1':
+            if key == 'h':
+                return h_mix_pQ(c.p.val_SI, 0, c.fluid_data, c.mixing_rule)
+
+        return super().initialise_source(c, key)
