@@ -12,7 +12,6 @@ SPDX-License-Identifier: MIT
 """
 
 import math
-import warnings
 
 import numpy as np
 
@@ -20,13 +19,14 @@ from tespy.components.component import Component
 from tespy.components.component import component_registry
 from tespy.tools import logger
 from tespy.tools.data_containers import ComponentCharacteristics as dc_cc
+from tespy.tools.data_containers import ComponentMandatoryConstraints as dc_cmc
 from tespy.tools.data_containers import ComponentProperties as dc_cp
 from tespy.tools.data_containers import GroupedComponentProperties as dc_gcp
 from tespy.tools.data_containers import SimpleDataContainer as dc_simple
-from tespy.tools.document_models import generate_latex_eq
+from tespy.tools.fluid_properties import h_mix_pT
 from tespy.tools.fluid_properties import s_mix_ph
 from tespy.tools.fluid_properties.helpers import darcy_friction_factor as dff
-from tespy.tools.helpers import convert_to_SI
+from tespy.tools.helpers import _numeric_deriv
 
 
 @component_registry
@@ -42,16 +42,17 @@ class SimpleHeatExchanger(Component):
 
     **Mandatory Equations**
 
-    - :py:meth:`tespy.components.component.Component.fluid_func`
-    - :py:meth:`tespy.components.component.Component.mass_flow_func`
+    - fluid: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
+    - mass flow: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
 
     **Optional Equations**
 
-    - :py:meth:`tespy.components.component.Component.pr_func`
+    - :py:meth:`tespy.components.component.Component.pr_structure_matrix`
+    - :py:meth:`tespy.components.component.Component.dp_structure_matrix`
     - :py:meth:`tespy.components.component.Component.zeta_func`
     - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.energy_balance_func`
-    - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.darcy_group_func`
-    - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.hw_group_func`
+    - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.darcy_func`
+    - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.hazen_williams_func`
     - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.kA_group_func`
     - :py:meth:`tespy.components.heat_exchangers.simple.SimpleHeatExchanger.kA_char_group_func`
 
@@ -154,14 +155,14 @@ class SimpleHeatExchanger(Component):
     >>> from tespy.components import Sink, Source, SimpleHeatExchanger
     >>> from tespy.connections import Connection
     >>> from tespy.networks import Network
-    >>> import shutil
-    >>> nw = Network()
-    >>> nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg', iterinfo=False)
+    >>> import os
+    >>> nw = Network(iterinfo=False)
+    >>> nw.units.set_defaults(**{
+    ...     "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+    ... })
     >>> so1 = Source('source 1')
     >>> si1 = Sink('sink 1')
     >>> heat_sink = SimpleHeatExchanger('heat sink')
-    >>> heat_sink.component()
-    'heat exchanger simple'
     >>> heat_sink.set_attr(Tamb=10, pr=0.95, design=['pr'],
     ... offdesign=['zeta', 'kA_char'])
     >>> inc = Connection(so1, 'out1', heat_sink, 'in1')
@@ -178,68 +179,145 @@ class SimpleHeatExchanger(Component):
     >>> inc.set_attr(fluid={'N2': 1}, m=1, T=200, p=5)
     >>> outg.set_attr(T=150, design=['T'])
     >>> nw.solve('design')
-    >>> nw.save('tmp')
+    >>> nw.save('tmp.json')
     >>> round(heat_sink.Q.val, 0)
     -52581.0
     >>> round(heat_sink.kA.val, 0)
     321.0
     >>> inc.set_attr(m=1.25)
-    >>> nw.solve('offdesign', design_path='tmp')
+    >>> nw.solve('offdesign', design_path='tmp.json')
     >>> round(heat_sink.Q.val, 0)
     -56599.0
     >>> round(outg.T.val, 1)
     156.9
     >>> inc.set_attr(m=0.75)
-    >>> nw.solve('offdesign', design_path='tmp')
+    >>> nw.solve('offdesign', design_path='tmp.json')
     >>> round(heat_sink.Q.val, 1)
     -47275.8
     >>> round(outg.T.val, 1)
     140.0
-    >>> shutil.rmtree('./tmp', ignore_errors=True)
+    >>> os.remove('tmp.json')
+
+    Use of the PowerConnection
+    --------------------------
+
+    Single sided heat exchangers can also connect to a
+    :code:`PowerConnection`. In this case the heat exchanger represents a heat
+    source from the perspective of the :code:`PowerConnection`, meaning, we
+    need a :code:`PowerSink` component as target of that connection.
+    To utilize the component in this connects, it is required to set the
+    :code:`power_connector_location`, in this case it should be the
+    :code:`'outlet'`.
+
+    >>> from tespy.connections import PowerConnection
+    >>> from tespy.components import PowerSink
+    >>> ambient = PowerSink('ambient heat dissipation')
+    >>> heat_sink.set_attr(power_connector_location='outlet')
+
+    Then we can create and add the :code:`PowerConnection`. We run a new
+    design calculation, because the old design case did not inlcude the
+    :code:`PowerConnection`. The energy value will be identical to the heat
+    transfer of the pipe.
+
+    >>> h1 = PowerConnection(heat_sink, 'heat', ambient, 'power', label='h1')
+    >>> nw.add_conns(h1)
+    >>> nw.solve('design')
+    >>> round(h1.E.val) == round(-heat_sink.Q.val)
+    True
     """
 
-    @staticmethod
-    def component():
-        return 'heat exchanger simple'
+    def get_mandatory_constraints(self):
+        constraints = super().get_mandatory_constraints()
+        if len(self.power_inl + self.power_outl) > 0:
+            constraints["energy_connector_balance"] = dc_cmc(**{
+                "func": self.energy_connector_balance_func,
+                "dependents": self.energy_connector_dependents,
+                "num_eq_sets": 1
+            })
+
+        return constraints
 
     def get_parameters(self):
         return {
+            'power_connector_location': dc_simple(),
             'Q': dc_cp(
-                deriv=self.energy_balance_deriv,
-                latex=self.energy_balance_func_doc, num_eq=1,
-                func=self.energy_balance_func),
+                num_eq_sets=1,
+                func=self.energy_balance_func,
+                dependents=self.energy_balance_dependents,
+                quantity="heat"
+            ),
             'pr': dc_cp(
-                min_val=1e-4, max_val=1, num_eq=1,
-                deriv=self.pr_deriv, latex=self.pr_func_doc,
-                func=self.pr_func, func_params={'pr': 'pr'}),
+                min_val=1e-4, max_val=1, num_eq_sets=1,
+                structure_matrix=self.pr_structure_matrix,
+                func_params={'pr': 'pr'},
+                quantity="ratio"
+            ),
+            'dp': dc_cp(
+                min_val=0, max_val=1e15, num_eq_sets=1,
+                structure_matrix=self.dp_structure_matrix,
+                func_params={'dp': 'dp'},
+                quantity="pressure"
+            ),
             'zeta': dc_cp(
-                min_val=0, max_val=1e15, num_eq=1,
-                deriv=self.zeta_deriv, func=self.zeta_func,
-                latex=self.zeta_func_doc,
-                func_params={'zeta': 'zeta'}),
-            'D': dc_cp(min_val=1e-2, max_val=2, d=1e-4),
-            'L': dc_cp(min_val=1e-1, d=1e-3),
-            'ks': dc_cp(val=1e-4, min_val=1e-7, max_val=1e-3, d=1e-8),
-            'ks_HW': dc_cp(val=10, min_val=1e-1, max_val=1e3, d=1e-2),
-            'kA': dc_cp(min_val=0, d=1),
-            'kA_char': dc_cc(param='m'), 'Tamb': dc_cp(),
-            'dissipative': dc_simple(val=None),
+                min_val=0, max_val=1e15, num_eq_sets=1,
+                func=self.zeta_func,
+                dependents=self.zeta_dependents,
+                func_params={'zeta': 'zeta'}
+            ),
+            'D': dc_cp(min_val=1e-2, max_val=2, d=1e-4, quantity="length"),
+            'L': dc_cp(min_val=1e-1, d=1e-3, quantity="length"),
+            'ks': dc_cp(
+                _val=1e-4, min_val=1e-7, max_val=1e-3, d=1e-8, quantity="length"
+            ),
+            'ks_HW': dc_cp(_val=10, min_val=1e-1, max_val=1e3, d=1e-2),
+            'kA': dc_cp(min_val=0, d=1, quantity="heat_transfer_coefficient"),
+            'kA_char': dc_cc(param='m'),
+            'Tamb': dc_cp(quantity="temperature"),
+            'dissipative': dc_simple(_val=None),
             'darcy_group': dc_gcp(
-                elements=['L', 'ks', 'D'], num_eq=1,
-                latex=self.darcy_func_doc,
-                func=self.darcy_func, deriv=self.darcy_deriv),
+                elements=['L', 'ks', 'D'], num_eq_sets=1,
+                func=self.darcy_func,
+                dependents=self.darcy_dependents
+            ),
             'hw_group': dc_gcp(
-                elements=['L', 'ks_HW', 'D'], num_eq=1,
-                latex=self.hazen_williams_func_doc,
-                func=self.hazen_williams_func, deriv=self.hazen_williams_deriv),
+                elements=['L', 'ks_HW', 'D'], num_eq_sets=1,
+                func=self.hazen_williams_func,
+                dependents=self.hazen_williams_dependents
+            ),
             'kA_group': dc_gcp(
-                elements=['kA', 'Tamb'], num_eq=1,
-                latex=self.kA_group_func_doc,
-                func=self.kA_group_func, deriv=self.kA_group_deriv),
+                elements=['kA', 'Tamb'], num_eq_sets=1,
+                func=self.kA_group_func,
+                dependents=self.kA_group_dependents
+            ),
             'kA_char_group': dc_gcp(
-                elements=['kA_char', 'Tamb'], num_eq=1,
-                latex=self.kA_char_group_func_doc,
-                func=self.kA_char_group_func, deriv=self.kA_char_group_deriv)
+                elements=['kA_char', 'Tamb'], num_eq_sets=1,
+                func=self.kA_char_group_func,
+                dependents=self.kA_char_group_dependents
+            )
+        }
+
+    def get_bypass_constraints(self):
+        return {
+            'mass_flow_constraints': dc_cmc(**{
+                'structure_matrix': self.variable_equality_structure_matrix,
+                'num_eq_sets': self.num_i,
+                'func_params': {'variable': 'm'}
+            }),
+            'pressure_constraints': dc_cmc(**{
+                'structure_matrix': self.variable_equality_structure_matrix,
+                'num_eq_sets': self.num_i,
+                'func_params': {'variable': 'p'}
+            }),
+            'enthalpy_constraints': dc_cmc(**{
+                'structure_matrix': self.variable_equality_structure_matrix,
+                'num_eq_sets': self.num_i,
+                'func_params': {'variable': 'h'}
+            }),
+            'fluid_constraints': dc_cmc(**{
+                'structure_matrix': self.variable_equality_structure_matrix,
+                'num_eq_sets': self.num_i,
+                'func_params': {'variable': 'fluid'}
+            })
         }
 
     @staticmethod
@@ -250,10 +328,38 @@ class SimpleHeatExchanger(Component):
     def outlets():
         return ['out1']
 
-    def preprocess(self, num_nw_vars):
-        super().preprocess(num_nw_vars)
+    def powerinlets(self):
+        if self.power_connector_location.val == "inlet":
+            return ['heat']
+        else:
+            return []
 
-        self.Tamb.val_SI = convert_to_SI('T', self.Tamb.val, self.inl[0].T.unit)
+    def poweroutlets(self):
+        if self.power_connector_location.val == "outlet":
+            return ['heat']
+        else:
+            return []
+
+    def _get_power_connector_location(self):
+        if self.power_connector_location.val == "inlet":
+            return self.power_inl[0]
+        else:
+            return self.power_outl[0]
+
+    def energy_connector_balance_func(self):
+        connector = self._get_power_connector_location()
+        if self.power_connector_location.val == "inlet":
+            energy_flow = -connector.E.val_SI
+        else:
+            energy_flow = connector.E.val_SI
+
+        return energy_flow + self.inl[0].m.val_SI * (
+                self.outl[0].h.val_SI - self.inl[0].h.val_SI
+            )
+
+    def energy_connector_dependents(self):
+        connector = self._get_power_connector_location()
+        return [connector.E, self.inl[0].m, self.outl[0].h, self.inl[0].h]
 
     def energy_balance_func(self):
         r"""
@@ -270,51 +376,15 @@ class SimpleHeatExchanger(Component):
         """
         return self.inl[0].m.val_SI * (
             self.outl[0].h.val_SI - self.inl[0].h.val_SI
-        ) - self.Q.val
+        ) - self.Q.val_SI
 
-    def energy_balance_func_doc(self, label):
-        r"""
-        Equation for pressure drop calculation.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0 = \dot{m}_\mathrm{in} \cdot \left(h_\mathrm{out} - '
-            r'h_\mathrm{in} \right) -\dot{Q}'
-        )
-        return generate_latex_eq(self, latex, label)
-
-    def energy_balance_deriv(self, increment_filter, k):
-        r"""
-        Calculate partial derivatives of energy balance.
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        i = self.inl[0]
-        o = self.outl[0]
-        if i.m.is_var:
-            self.jacobian[k, i.m.J_col] = o.h.val_SI - i.h.val_SI
-        if i.h.is_var:
-            self.jacobian[k, i.h.J_col] = -i.m.val_SI
-        if o.h.is_var:
-            self.jacobian[k, o.h.J_col] = i.m.val_SI
-        # custom variable Q
-        if self.Q.is_var:
-            self.jacobian[k, self.Q.J_col] = -1
+    def energy_balance_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].h,
+            self.outl[0].h,
+            self.Q,
+        ]
 
     def darcy_func(self):
         r"""
@@ -347,74 +417,26 @@ class SimpleHeatExchanger(Component):
         visc_o = o.calc_viscosity(T0=o.T.val_SI)
         v_i = i.calc_vol(T0=i.T.val_SI)
         v_o = o.calc_vol(T0=o.T.val_SI)
-
-        Re = 4 * abs(i.m.val_SI) / (math.pi * self.D.val * (visc_i + visc_o) / 2)
+        Re = (
+            4 * abs(i.m.val_SI)
+            / (math.pi * self.D.val_SI * (visc_i + visc_o) / 2)
+        )
 
         return (
             (i.p.val_SI - o.p.val_SI)
             - 8 * abs(i.m.val_SI) * i.m.val_SI * (v_i + v_o)
-            / 2 * self.L.val * dff(Re, self.ks.val, self.D.val)
-            / (math.pi ** 2 * self.D.val ** 5)
+            / 2 * self.L.val_SI * dff(Re, self.ks.val_SI, self.D.val_SI)
+            / (math.pi ** 2 * self.D.val_SI ** 5)
         )
 
-    def darcy_func_doc(self, label):
-        r"""
-        Equation for pressure drop calculation from darcy friction factor.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'\begin{split}' + '\n'
-            r'0 = &p_\mathrm{in}-p_\mathrm{out}-'
-            r'\frac{8\cdot|\dot{m}_\mathrm{in}| \cdot\dot{m}_\mathrm{in}'
-            r'\cdot \frac{v_\mathrm{in}+v_\mathrm{out}}{2} \cdot L \cdot'
-            r'\lambda\left(Re, ks, D\right)}{\pi^2 \cdot D^5}\\' + '\n'
-            r'Re =&\frac{4 \cdot |\dot{m}_\mathrm{in}|}{\pi \cdot D \cdot'
-            r'\frac{\eta_\mathrm{in}+\eta_\mathrm{out}}{2}}\\' + '\n'
-            r'\end{split}'
-        )
-        return generate_latex_eq(self, latex, label)
-
-    def darcy_deriv(self, increment_filter, k):
-        r"""
-        Calculate partial derivatives of hydro group (pressure drop).
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        func = self.darcy_func
-        i = self.inl[0]
-        o = self.outl[0]
-        if self.is_variable(i.m, increment_filter):
-            self.jacobian[k, i.m.J_col] = self.numeric_deriv(func, 'm', i)
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(func, 'p', i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(func, 'h', i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(func, 'p', o)
-        if self.is_variable(o.h, increment_filter):
-            self.jacobian[k, o.h.J_col] = self.numeric_deriv(func, 'h', o)
-        # custom variables of hydro group
-        for variable_name in self.darcy_group.elements:
-            parameter = self.get_attr(variable_name)
-            if parameter.is_var:
-                self.jacobian[k, parameter.J_col] = (
-                    self.numeric_deriv(func, variable_name, None)
-                )
+    def darcy_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+        ] + [self.get_attr(element) for element in self.darcy_group.elements]
 
     def hazen_williams_func(self):
         r"""
@@ -453,65 +475,82 @@ class SimpleHeatExchanger(Component):
         return (
             math.copysign(i.p.val_SI - o.p.val_SI, i.m.val_SI)
             - (
-                10.67 * abs(i.m.val_SI) ** 1.852 * self.L.val /
-                (self.ks_HW.val ** 1.852 * self.D.val ** 4.871)
+                10.67 * abs(i.m.val_SI) ** 1.852 * self.L.val_SI /
+                (self.ks_HW.val_SI ** 1.852 * self.D.val_SI ** 4.871)
             ) * (9.81 * ((v_i + v_o) / 2) ** 0.852)
         )
 
-    def hazen_williams_func_doc(self, label):
-        r"""
-        Equation for pressure drop calculation from Hazen-Williams equation.
+    def hazen_williams_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+        ] + [self.get_attr(element) for element in self.hw_group.elements]
 
-        Parameters
-        ----------
-        label : str
-            Label for equation.
+    def _calculate_td_log(self):
+        r"""
+        Calculation of mean logarithmic temperature difference.
+
+        For numerical stability: If temperature differences have
+        different sign use mean difference to avoid negative logarithm.
 
         Returns
         -------
-        latex : str
-            LaTeX code of equations applied.
+        deltaT_log : float
+            Mean logarithmic temperature difference.
+
+            .. math::
+
+                \Delta T_{log} = \begin{cases}
+                \frac{T_{in}-T_{out}}{\ln{\frac{T_{in}-T_{amb}}
+                {T_{out}-T_{amb}}}} & T_{in} > T_{out} \\
+                \frac{T_{out}-T_{in}}{\ln{\frac{T_{out}-T_{amb}}
+                {T_{in}-T_{amb}}}} & T_{in} < T_{out}\\
+                0 & T_{in} = T_{out}
+                \end{cases}
+
+                T_{amb}: \text{ambient temperature}
         """
-        latex = (
-            r'0 = \left(p_\mathrm{in} - p_\mathrm{out} \right) -'
-            r'\frac{10.67 \cdot |\dot{m}_\mathrm{in}| ^ {1.852}'
-            r'\cdot L}{ks^{1.852} \cdot D^{4.871}} \cdot g \cdot'
-            r'\left(\frac{v_\mathrm{in}+ v_\mathrm{out}}{2}\right)^{0.852}'
-        )
-        return generate_latex_eq(self, latex, label)
-
-    def hazen_williams_deriv(self, increment_filter, k):
-        r"""
-        Calculate partial derivatives of hydro group (pressure drop).
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        func = self.hazen_williams_func
         i = self.inl[0]
         o = self.outl[0]
-        if self.is_variable(i.m, increment_filter):
-            self.jacobian[k, i.m.J_col] = self.numeric_deriv(func, 'm', i)
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(func, 'p', i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(func, 'h', i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(func, 'p', o)
-        if self.is_variable(o.h, increment_filter):
-            self.jacobian[k, o.h.J_col] = self.numeric_deriv(func, 'h', o)
-        # custom variables of hydro group
-        for variable_name in self.hw_group.elements:
-            parameter = self.get_attr(variable_name)
-            if parameter.is_var:
-                self.jacobian[k, parameter.J_col] = (
-                    self.numeric_deriv(func, variable_name, None)
-                )
+
+        ttd_1 = i.calc_T() - self.Tamb.val_SI
+        ttd_2 = o.calc_T() - self.Tamb.val_SI
+
+        # For numerical stability: If temperature differences have
+        # different sign use mean difference to avoid negative logarithm.
+        if (ttd_1 / ttd_2) < 0:
+            if ttd_1 > 0:
+                if o.h.is_var and self.it < 10:
+                    h_out = h_mix_pT(
+                        o.p.val_SI,
+                        self.Tamb.val_SI + 0.0001,
+                        o.fluid_data,
+                        o.mixing_rule
+                    )
+                    o.h.set_reference_val_SI(h_out)
+                ttd_2 = 0.1
+            elif ttd_1 < 0:
+                if o.h.is_var and self.it < 10:
+                    h_out = h_mix_pT(
+                        o.p.val_SI,
+                        self.Tamb.val_SI - 0.0001,
+                        o.fluid_data,
+                        o.mixing_rule
+                    )
+                    o.h.set_reference_val_SI(h_out)
+                ttd_2 = -0.1
+
+        if round(ttd_1, 6) == round(ttd_2, 6):
+            td_log = ttd_2
+        elif ttd_1 > ttd_2:
+            td_log = (ttd_1 - ttd_2) / math.log(ttd_1 / ttd_2)
+        else:
+            td_log = (ttd_2 - ttd_1) / math.log(ttd_2 / ttd_1)
+
+        return td_log
 
     def kA_group_func(self):
         r"""
@@ -539,83 +578,18 @@ class SimpleHeatExchanger(Component):
         """
         i = self.inl[0]
         o = self.outl[0]
+        td_log = self._calculate_td_log()
+        return i.m.val_SI * (o.h.val_SI - i.h.val_SI) + self.kA.val_SI * td_log
 
-        ttd_1 = i.calc_T() - self.Tamb.val_SI
-        ttd_2 = o.calc_T() - self.Tamb.val_SI
-
-        # For numerical stability: If temperature differences have
-        # different sign use mean difference to avoid negative logarithm.
-        if (ttd_1 / ttd_2) < 0:
-            td_log = (ttd_2 + ttd_1) / 2
-        elif ttd_1 > ttd_2:
-            td_log = (ttd_1 - ttd_2) / math.log(ttd_1 / ttd_2)
-        elif ttd_1 < ttd_2:
-            td_log = (ttd_2 - ttd_1) / math.log(ttd_2 / ttd_1)
-        else:
-            # both values are equal
-            td_log = ttd_2
-
-        return i.m.val_SI * (o.h.val_SI - i.h.val_SI) + self.kA.val * td_log
-
-    def kA_group_func_doc(self, label):
-        r"""
-        Calculate heat transfer from heat transfer coefficient.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'\begin{split}' + '\n'
-            r'0=&\dot{m}_\mathrm{in}\cdot\left(h_\mathrm{out}-'
-            r'h_\mathrm{in}\right)+kA \cdot \Delta T_\mathrm{log}\\' + '\n'
-            r'\Delta T_\mathrm{log} = &\begin{cases}' + '\n'
-            r'\frac{T_\mathrm{in}-T_\mathrm{out}}{\ln{\frac{T_\mathrm{in}-'
-            r'T_\mathrm{amb}}{T_\mathrm{out}-T_\mathrm{amb}}}} &'
-            r' T_\mathrm{in} > T_\mathrm{out} \\' + '\n'
-            r'\frac{T_\mathrm{out}-T_\mathrm{in}}{\ln{\frac{'
-            r'T_\mathrm{out}-T_\mathrm{amb}}{T_\mathrm{in}-'
-            r'T_\mathrm{amb}}}} & T_\mathrm{in} < T_\mathrm{out}\\' + '\n'
-            r'0 & T_\mathrm{in} = T_\mathrm{out}' + '\n'
-            r'\end{cases}\\' + '\n'
-            r'T_\mathrm{amb} =& \text{ambient temperature}' + '\n'
-            r'\end{split}'
-        )
-        return generate_latex_eq(self, latex, label)
-
-    def kA_group_deriv(self, increment_filter, k):
-        r"""
-        Calculate partial derivatives of kA group.
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        f = self.kA_group_func
-        i = self.inl[0]
-        o = self.outl[0]
-        if self.is_variable(i.m, increment_filter):
-            self.jacobian[k, i.m.J_col] = o.h.val_SI - i.h.val_SI
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(f, 'p', i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(f, 'h', i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(f, 'p', o)
-        if self.is_variable(o.h, increment_filter):
-            self.jacobian[k, o.h.J_col] = self.numeric_deriv(f, 'h', o)
-        if self.kA.is_var:
-            self.jacobian[k, self.kA.J_col] = self.numeric_deriv(f, self.vars[self.kA])
+    def kA_group_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+            self.kA
+        ]
 
     def kA_char_group_func(self):
         r"""
@@ -646,92 +620,57 @@ class SimpleHeatExchanger(Component):
         Note
         ----
         For standard function of f\ :subscript:`kA` \ see module
-        :py:mod:`tespy.data`.
+        :ref:`tespy.data <tespy_data_label>`.
         """
-        p = self.kA_char.param
-        expr = self.get_char_expr(p, **self.kA_char.char_params)
         i = self.inl[0]
         o = self.outl[0]
+        p = self.kA_char.param
 
-        # For numerical stability: If temperature differences have
-        # different sign use mean difference to avoid negative logarithm.
-
-        ttd_1 = i.calc_T() - self.Tamb.val_SI
-        ttd_2 = o.calc_T() - self.Tamb.val_SI
-
-        if (ttd_1 / ttd_2) < 0:
-            td_log = (ttd_2 + ttd_1) / 2
-        elif ttd_1 > ttd_2:
-            td_log = (ttd_1 - ttd_2) / math.log(ttd_1 / ttd_2)
-        elif ttd_1 < ttd_2:
-            td_log = (ttd_2 - ttd_1) / math.log(ttd_2 / ttd_1)
-        else:
-            # both values are equal
-            td_log = ttd_2
-
+        expr = self.get_char_expr(p, **self.kA_char.char_params)
         fkA = 2 / (1 + 1 / self.kA_char.char_func.evaluate(expr))
 
-        return i.m.val_SI * (o.h.val_SI - i.h.val_SI) + self.kA.design * fkA * td_log
+        td_log = self._calculate_td_log()
 
-    def kA_char_group_func_doc(self, label):
-        r"""
-        Calculate heat transfer from heat transfer coefficient characteristic.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'\begin{split}' + '\n'
-            r'0=&\dot{m}_\mathrm{in}\cdot\left(h_\mathrm{out}-'
-            r'h_\mathrm{in}\right)+kA_\mathrm{design} \cdot f_\mathrm{kA}'
-            r' \cdot \Delta T_\mathrm{log}\\' + '\n'
-            r'\Delta T_\mathrm{log} = &\begin{cases}' + '\n'
-            r'\frac{T_\mathrm{in}-T_\mathrm{out}}{\ln{\frac{T_\mathrm{in}-'
-            r'T_\mathrm{amb}}{T_\mathrm{out}-T_\mathrm{amb}}}} &'
-            r' T_\mathrm{in} > T_\mathrm{out} \\' + '\n'
-            r'\frac{T_\mathrm{out}-T_\mathrm{in}}{\ln{\frac{'
-            r'T_\mathrm{out}-T_\mathrm{amb}}{T_\mathrm{in}-'
-            r'T_\mathrm{amb}}}} & T_\mathrm{in} < T_\mathrm{out}\\' + '\n'
-            r'0 & T_\mathrm{in} = T_\mathrm{out}' + '\n'
-            r'\end{cases}\\' + '\n'
-            r'f_{kA}=&\frac{2}{1 + \frac{1}{f\left(X\right)}}\\' + '\n'
-            r'T_\mathrm{amb} =& \text{ambient temperature}' + '\n'
-            r'\end{split}'
+        return (
+            i.m.val_SI * (o.h.val_SI - i.h.val_SI)
+            + self.kA.design * fkA * td_log
         )
-        return generate_latex_eq(self, latex, label)
 
-    def kA_char_group_deriv(self, increment_filter, k):
-        r"""
-        Calculate partial derivatives of kA characteristics.
+    def kA_char_group_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+        ]
 
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        f = self.kA_char_group_func
-        i = self.inl[0]
-        o = self.outl[0]
-        if self.is_variable(i.m, increment_filter):
-            self.jacobian[k, i.m.J_col] = self.numeric_deriv(f, 'm', i)
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(f, 'p', i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(f, 'h', i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(f, 'p', o)
-        if self.is_variable(o.h, increment_filter):
-            self.jacobian[k, o.h.J_col] = self.numeric_deriv(f, 'h', o)
+    def convergence_check(self):
+        if self.kA_group.is_set:
+            i = self.inl[0]
+            o = self.outl[0]
+            T_in = i.calc_T()
+            T_out = o.calc_T()
+            if T_in > self.Tamb.val_SI:
+                if T_out < self.Tamb.val_SI:
+                    if o.h.is_var:
+                        h_out = h_mix_pT(
+                            o.p.val_SI,
+                            self.Tamb.val_SI + 0.0001,
+                            o.fluid_data,
+                            o.mixing_rule
+                        )
+                        o.h.set_reference_val_SI(h_out)
+            elif T_in < self.Tamb.val_SI:
+                if T_out > self.Tamb.val_SI:
+                    if o.h.is_var:
+                        h_out = h_mix_pT(
+                            o.p.val_SI,
+                            self.Tamb.val_SI - 0.0001,
+                            o.fluid_data,
+                            o.mixing_rule
+                        )
+                        o.h.set_reference_val_SI(h_out)
 
     def bus_func(self, bus):
         r"""
@@ -755,25 +694,8 @@ class SimpleHeatExchanger(Component):
                 \dot{E} = \dot{m}_{in} \cdot \left( h_{out} - h_{in} \right)
         """
         return self.inl[0].m.val_SI * (
-            self.outl[0].h.val_SI - self.inl[0].h.val_SI)
-
-    def bus_func_doc(self, bus):
-        r"""
-        Return LaTeX string of the bus function.
-
-        Parameters
-        ----------
-        bus : tespy.connections.bus.Bus
-            TESPy bus object.
-
-        Returns
-        -------
-        latex : str
-            LaTeX string of bus function.
-        """
-        return (
-            r'\dot{m}_\mathrm{in} \cdot \left(h_\mathrm{out} - '
-            r'h_\mathrm{in} \right)')
+            self.outl[0].h.val_SI - self.inl[0].h.val_SI
+        )
 
     def bus_deriv(self, bus):
         r"""
@@ -793,17 +715,17 @@ class SimpleHeatExchanger(Component):
         if self.inl[0].m.is_var:
             if self.inl[0].m.J_col not in bus.jacobian:
                 bus.jacobian[self.inl[0].m.J_col] = 0
-            bus.jacobian[self.inl[0].m.J_col] -= self.numeric_deriv(f, 'm', self.inl[0], bus=bus)
+            bus.jacobian[self.inl[0].m.J_col] -= _numeric_deriv(self.inl[0].m._reference_container, f, bus=bus)
 
         if self.inl[0].h.is_var:
             if self.inl[0].h.J_col not in bus.jacobian:
                 bus.jacobian[self.inl[0].h.J_col] = 0
-            bus.jacobian[self.inl[0].h.J_col] -= self.numeric_deriv(f, 'h', self.inl[0], bus=bus)
+            bus.jacobian[self.inl[0].h.J_col] -= _numeric_deriv(self.inl[0].h._reference_container, f, bus=bus)
 
         if self.outl[0].h.is_var:
             if self.outl[0].h.J_col not in bus.jacobian:
                 bus.jacobian[self.outl[0].h.J_col] = 0
-            bus.jacobian[self.outl[0].h.J_col] -= self.numeric_deriv(f, 'h', self.outl[0], bus=bus)
+            bus.jacobian[self.outl[0].h.J_col] -= _numeric_deriv(self.outl[0].h._reference_container, f, bus=bus)
 
     def initialise_source(self, c, key):
         r"""
@@ -887,25 +809,19 @@ class SimpleHeatExchanger(Component):
         i = self.inl[0]
         o = self.outl[0]
 
-        self.Q.val = i.m.val_SI * (o.h.val_SI - i.h.val_SI)
-        self.pr.val = o.p.val_SI / i.p.val_SI
-        self.zeta.val = self.calc_zeta(i, o)
+        self.Q.val_SI = i.m.val_SI * (o.h.val_SI - i.h.val_SI)
+        self.pr.val_SI = o.p.val_SI / i.p.val_SI
+        self.dp.val_SI = i.p.val_SI - o.p.val_SI
+        self.zeta.val_SI = self.calc_zeta(i, o)
 
         if self.Tamb.is_set:
             ttd_1 = i.T.val_SI - self.Tamb.val_SI
             ttd_2 = o.T.val_SI - self.Tamb.val_SI
-
-            if (ttd_1 / ttd_2) < 0:
-                td_log = np.nan
-            if ttd_1 > ttd_2:
-                td_log = (ttd_1 - ttd_2) / math.log(ttd_1 / ttd_2)
-            elif ttd_1 < ttd_2:
-                td_log = (ttd_2 - ttd_1) / math.log(ttd_2 / ttd_1)
+            if ttd_1 / ttd_2 < 0:
+                self.kA.val_SI = np.nan
             else:
-                # both values are equal
-                td_log = ttd_1
+                self.kA.val_SI = abs(self.Q.val_SI / self._calculate_td_log())
 
-            self.kA.val = abs(self.Q.val / td_log)
             self.kA.is_result = True
         else:
             self.kA.is_result = False
@@ -1053,7 +969,7 @@ class SimpleHeatExchanger(Component):
                 f"exergy analysis for component {self.label}."
             )
             logger.warning(msg)
-        if self.Q.val < 0:
+        if self.Q.val_SI < 0:
             if self.inl[0].T.val_SI >= T0 and self.outl[0].T.val_SI >= T0:
                 if self.dissipative.val:
                     self.E_P = np.nan
@@ -1088,7 +1004,7 @@ class SimpleHeatExchanger(Component):
                 self.E_bus = {
                     "chemical": np.nan, "physical": np.nan, "massless": np.nan
                 }
-        elif self.Q.val > 0:
+        elif self.Q.val_SI > 0:
             if self.inl[0].T.val_SI >= T0 - 1e-6 and self.outl[0].T.val_SI >= T0 - 1e-6:
                 self.E_P = self.outl[0].Ex_physical - self.inl[0].Ex_physical
                 self.E_F = self.outl[0].Ex_therm - self.inl[0].Ex_therm
@@ -1160,14 +1076,3 @@ class SimpleHeatExchanger(Component):
                 'ending_point_value': self.outl[0].s.val
             }
         }
-
-
-class HeatExchangerSimple(SimpleHeatExchanger):
-
-    def __init__(self, label, **kwargs):
-        super().__init__(label, **kwargs)
-        msg = (
-            "The API for the component HeatExchangerSimple will change with "
-            "the next major release, please import SimpleHeatExchanger instead."
-        )
-        warnings.warn(msg, FutureWarning)
