@@ -16,10 +16,14 @@ from tespy.components.component import component_registry
 from tespy.components.turbomachinery.base import Turbomachine
 from tespy.tools import logger
 from tespy.tools.data_containers import ComponentCharacteristics as dc_cc
+from tespy.tools.data_containers import ComponentMandatoryConstraints as dc_cmc
 from tespy.tools.data_containers import ComponentProperties as dc_cp
 from tespy.tools.data_containers import SimpleDataContainer as dc_simple
-from tespy.tools.document_models import generate_latex_eq
+from tespy.tools.fluid_properties import h_mix_pQ
+from tespy.tools.fluid_properties import h_mix_pT
 from tespy.tools.fluid_properties import isentropic
+from tespy.tools.fluid_properties import single_fluid
+from tespy.tools.helpers import _get_dependents
 
 
 @component_registry
@@ -29,12 +33,13 @@ class Turbine(Turbomachine):
 
     **Mandatory Equations**
 
-    - :py:meth:`tespy.components.component.Component.fluid_func`
-    - :py:meth:`tespy.components.component.Component.mass_flow_func`
+    - fluid: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
+    - mass flow: :py:meth:`tespy.components.component.Component.variable_equality_structure_matrix`
 
     **Optional Equations**
 
-    - :py:meth:`tespy.components.component.Component.pr_func`
+    - :py:meth:`tespy.components.component.Component.dp_structure_matrix`
+    - :py:meth:`tespy.components.component.Component.pr_structure_matrix`
     - :py:meth:`tespy.components.turbomachinery.base.Turbomachine.energy_balance_func`
     - :py:meth:`tespy.components.turbomachinery.turbine.Turbine.eta_s_func`
     - :py:meth:`tespy.components.turbomachinery.turbine.Turbine.eta_s_char_func`
@@ -45,6 +50,10 @@ class Turbine(Turbomachine):
     - in1
     - out1
 
+    Optional outlets
+
+    - power
+
     Image
 
     .. image:: /api/_images/Turbine.svg
@@ -89,9 +98,13 @@ class Turbine(Turbomachine):
     eta_s : float, dict
         Isentropic efficiency, :math:`\eta_s/1`
 
-    pr : float, dict, :code:`"var"`
+    pr : float, dict
         Outlet to inlet pressure ratio, :math:`pr/1`
 
+    dp : float, dict
+        Inlet to outlet pressure difference, :math:`dp/\text{p}_\text{unit}`
+        Is specified in the Network's pressure unit
+
     eta_s_char : tespy.tools.characteristics.CharLine, dict
         Characteristic curve for isentropic efficiency, provide CharLine as
         function :code:`func`.
@@ -109,13 +122,15 @@ class Turbine(Turbomachine):
     >>> from tespy.connections import Connection
     >>> from tespy.networks import Network
     >>> from tespy.tools import ComponentCharacteristics as dc_cc
-    >>> import shutil
-    >>> nw = Network(p_unit='bar', T_unit='C', h_unit='kJ / kg', iterinfo=False)
+    >>> import os
+    >>> nw = Network(iterinfo=False)
+    >>> nw.units.set_defaults(**{
+    ...     "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg",
+    ...     "mass_flow": "t/h"
+    ... })
     >>> si = Sink('sink')
     >>> so = Source('source')
     >>> t = Turbine('turbine')
-    >>> t.component()
-    'turbine'
     >>> inc = Connection(so, 'out1', t, 'in1')
     >>> outg = Connection(t, 'out1', si, 'in1')
     >>> nw.add_conns(inc, outg)
@@ -126,51 +141,85 @@ class Turbine(Turbomachine):
 
     >>> t.set_attr(eta_s=0.9, design=['eta_s'],
     ... offdesign=['eta_s_char', 'cone'])
-    >>> inc.set_attr(fluid={'water': 1}, m=10, T=550, p=110, design=['p'])
+    >>> inc.set_attr(fluid={'water': 1}, m=36, T=550, p=110, design=['p'])
     >>> outg.set_attr(p=0.5)
     >>> nw.solve('design')
-    >>> nw.save('tmp')
+    >>> nw.save('tmp.json')
     >>> round(t.P.val, 0)
     -10452574.0
     >>> round(outg.x.val, 3)
     0.914
-    >>> inc.set_attr(m=8)
-    >>> nw.solve('offdesign', design_path='tmp')
+    >>> inc.set_attr(m=28.8)
+    >>> nw.solve('offdesign', design_path='tmp.json')
     >>> round(t.eta_s.val, 3)
     0.898
     >>> round(inc.p.val, 1)
     88.6
-    >>> shutil.rmtree('./tmp', ignore_errors=True)
+    >>> os.remove('tmp.json')
     """
 
     @staticmethod
-    def component():
-        return 'turbine'
+    def poweroutlets():
+        return ["power"]
+
+    def get_mandatory_constraints(self):
+        constraints = super().get_mandatory_constraints()
+        if len(self.power_outl) > 0:
+            constraints["energy_connector_balance"] = dc_cmc(**{
+                "func": self.energy_connector_balance_func,
+                "dependents": self.energy_connector_dependents,
+                "num_eq_sets": 1
+            })
+
+        return constraints
+
+    def energy_connector_balance_func(self):
+        r"""
+        (optional) energy balance equation connecting the power connector to
+        the component's power
+
+        Returns
+        -------
+        residual : float
+            Residual value of equation
+
+            .. math::
+
+                0=\dot E + \dot{m}_{in}\cdot\left(h_{out}-h_{in}\right)
+        """
+        return self.power_outl[0].E.val_SI + self.inl[0].m.val_SI * (
+            self.outl[0].h.val_SI - self.inl[0].h.val_SI
+        )
+
+    def energy_connector_dependents(self):
+        return [self.power_outl[0].E, self.inl[0].m, self.outl[0].h, self.inl[0].h]
 
     def get_parameters(self):
-        return {
-            'P': dc_cp(
-                max_val=0, num_eq=1,
-                deriv=self.energy_balance_deriv,
-                func=self.energy_balance_func,
-                latex=self.energy_balance_func_doc),
+        parameters = super().get_parameters()
+        parameters["P"].max_val = 0
+        parameters["pr"].max_val = 1
+        parameters["pr"].min_val = 0
+        parameters["dp"].min_val = 0
+        parameters.update({
             'eta_s': dc_cp(
-                min_val=0, max_val=1, num_eq=1,
+                min_val=0, max_val=1, num_eq_sets=1,
+                func=self.eta_s_func,
+                dependents=self.eta_s_dependents,
                 deriv=self.eta_s_deriv,
-                func=self.eta_s_func, latex=self.eta_s_func_doc),
+                quantity="efficiency"
+            ),
             'eta_s_char': dc_cc(
-                param='m', num_eq=1,
-                deriv=self.eta_s_char_deriv,
-                func=self.eta_s_char_func, latex=self.eta_s_char_func_doc),
-            'pr': dc_cp(
-                min_val=0, max_val=1, num_eq=1,
-                deriv=self.pr_deriv,
-                func=self.pr_func, func_params={'pr': 'pr'},
-                latex=self.pr_func_doc),
+                param='m', num_eq_sets=1,
+                func=self.eta_s_char_func,
+                dependents=self.eta_s_char_dependents
+            ),
             'cone': dc_simple(
-                deriv=self.cone_deriv, num_eq=1,
-                func=self.cone_func, latex=self.cone_func_doc)
-        }
+                num_eq_sets=1,
+                func=self.cone_func,
+                dependents=self.cone_dependents
+            )
+        })
+        return parameters
 
     def eta_s_func(self):
         r"""
@@ -200,29 +249,10 @@ class Turbine(Turbomachine):
                     T0=inl.T.val_SI
                 )
                 - inl.h.val_SI
-            ) * self.eta_s.val
+            ) * self.eta_s.val_SI
         )
 
-    def eta_s_func_doc(self, label):
-        r"""
-        Equation for given isentropic efficiency of a turbine.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0=-\left(h_\mathrm{out}-h_\mathrm{in}\right)+\left('
-            r'h_\mathrm{out,s}-h_\mathrm{in}\right)\cdot\eta_\mathrm{s}')
-        return generate_latex_eq(self, latex, label)
-
-    def eta_s_deriv(self, increment_filter, k):
+    def eta_s_deriv(self, increment_filter, k, dependents=None):
         r"""
         Partial derivatives for isentropic efficiency function.
 
@@ -234,17 +264,26 @@ class Turbine(Turbomachine):
         k : int
             Position of derivatives in Jacobian matrix (k-th equation).
         """
+        dependents = dependents["scalars"][0]
         f = self.eta_s_func
         i = self.inl[0]
         o = self.outl[0]
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(f, "p", i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(f, "p", o)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(f, "h", i)
-        if o.h.is_var and self.it == 0:
-            self.jacobian[k, o.h.J_col] = -1
+
+        if o.h.is_var and not i.h.is_var:
+            self._partial_derivative(o.h, k, -1, increment_filter)
+            # remove o.h from the dependents
+            dependents = dependents.difference(_get_dependents([o.h])[0])
+
+        for dependent in dependents:
+            self._partial_derivative(dependent, k, f, increment_filter)
+
+    def eta_s_dependents(self):
+        return [
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+        ]
 
     def cone_func(self):
         r"""
@@ -267,7 +306,7 @@ class Turbine(Turbomachine):
         i = self.inl[0]
         o = self.outl[0]
         vol = i.calc_vol(T0=i.T.val_SI)
-        return (
+        residual = (
             - i.m.val_SI + i.m.design * i.p.val_SI / i.p.design
             * (i.p.design * i.vol.design / (i.p.val_SI * vol)) ** 0.5
             * abs(
@@ -275,54 +314,15 @@ class Turbine(Turbomachine):
                     / (1 - (self.pr.design) ** ((n + 1) / n))
             ) ** 0.5
         )
+        return residual
 
-    def cone_func_doc(self, label):
-        r"""
-        Equation for stodolas cone law.
-
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0 = \frac{\dot{m}_\mathrm{in,design}\cdot p_\mathrm{in}}'
-            r'{p_\mathrm{in,design}}\cdot\sqrt{\frac{p_\mathrm{in,design}'
-            r'\cdot v_\mathrm{in}}{p_\mathrm{in}\cdot '
-            r'v_\mathrm{in,design}}\cdot\frac{1-\left('
-            r'\frac{p_\mathrm{out}}{p_\mathrm{in}} \right)^{2}}'
-            r'{1-\left(\frac{p_\mathrm{out,design}}{p_\mathrm{in,design}}'
-            r'\right)^{2}}} -\dot{m}_\mathrm{in}')
-        return generate_latex_eq(self, latex, label)
-
-    def cone_deriv(self, increment_filter, k):
-        r"""
-        Partial derivatives for stodolas cone law.
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        f = self.cone_func
-        i = self.inl[0]
-        o = self.outl[0]
-        if i.m.is_var:
-            self.jacobian[k, i.m.J_col] = -1
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(f, 'p', i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(f, 'h', i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(f, 'p', o)
+    def cone_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+        ]
 
     def eta_s_char_func(self):
         r"""
@@ -366,51 +366,30 @@ class Turbine(Turbomachine):
             )
         )
 
-    def eta_s_char_func_doc(self, label):
-        r"""
-        Equation for given isentropic efficiency characteristic.
+    def eta_s_char_dependents(self):
+        return [
+            self.inl[0].m,
+            self.inl[0].p,
+            self.inl[0].h,
+            self.outl[0].p,
+            self.outl[0].h,
+        ]
 
-        Parameters
-        ----------
-        label : str
-            Label for equation.
-
-        Returns
-        -------
-        latex : str
-            LaTeX code of equations applied.
-        """
-        latex = (
-            r'0=-\left(h_\mathrm{out}-h_\mathrm{in}\right)+'
-            r'\eta_\mathrm{s,design}\cdot f \left(X\right)'
-            r'\cdot\left(h_\mathrm{out,s}-h_\mathrm{in}\right)')
-        return generate_latex_eq(self, latex, label)
-
-    def eta_s_char_deriv(self, increment_filter, k):
-        r"""
-        Partial derivatives for isentropic efficiency characteristic.
-
-        Parameters
-        ----------
-        increment_filter : ndarray
-            Matrix for filtering non-changing variables.
-
-        k : int
-            Position of derivatives in Jacobian matrix (k-th equation).
-        """
-        f = self.eta_s_char_func
-        i = self.inl[0]
-        o = self.outl[0]
-        if self.is_variable(i.m, increment_filter):
-            self.jacobian[k, i.m.J_col] = self.numeric_deriv(f, 'm', i)
-        if self.is_variable(i.p, increment_filter):
-            self.jacobian[k, i.p.J_col] = self.numeric_deriv(f, "p", i)
-        if self.is_variable(i.h, increment_filter):
-            self.jacobian[k, i.h.J_col] = self.numeric_deriv(f, "h", i)
-        if self.is_variable(o.p, increment_filter):
-            self.jacobian[k, o.p.J_col] = self.numeric_deriv(f, "p", o)
-        if self.is_variable(o.h, increment_filter):
-            self.jacobian[k, o.h.J_col] = self.numeric_deriv(f, "h", o)
+    def calc_eta_s(self):
+        inl = self.inl[0]
+        outl = self.outl[0]
+        return (
+            (outl.h.val_SI - inl.h.val_SI)
+            / (isentropic(
+                    inl.p.val_SI,
+                    inl.h.val_SI,
+                    outl.p.val_SI,
+                    inl.fluid_data,
+                    inl.mixing_rule,
+                    T0=inl.T.val_SI
+                ) - inl.h.val_SI
+            )
+        )
 
     def convergence_check(self):
         r"""
@@ -423,24 +402,14 @@ class Turbine(Turbomachine):
         """
         i, o = self.inl[0], self.outl[0]
 
-        if not i.good_starting_values:
-            if i.p.val_SI <= 1e5 and i.p.is_var:
-                i.p.val_SI = 1e5
-
-            if i.h.val_SI < 10e5 and i.h.is_var:
-                i.h.val_SI = 10e5
-
-            if o.h.val_SI < 5e5 and o.h.is_var:
-                o.h.val_SI = 5e5
-
         if i.h.val_SI <= o.h.val_SI and o.h.is_var:
-            o.h.val_SI = i.h.val_SI * 0.9
+            o.h.set_reference_val_SI(i.h.val_SI - 100e3)
 
         if i.p.val_SI <= o.p.val_SI and o.p.is_var:
-            o.p.val_SI = i.p.val_SI * 0.9
+            o.p.set_reference_val_SI(i.p.val_SI * 2 /3)
 
     @staticmethod
-    def initialise_Source(c, key):
+    def initialise_source(c, key):
         r"""
         Return a starting value for pressure and enthalpy at outlet.
 
@@ -456,18 +425,24 @@ class Turbine(Turbomachine):
         -------
         val : float
             Starting value for pressure/enthalpy in SI units.
-
-            .. math::
-
-                val = \begin{cases}
-                5 \cdot 10^4 & \text{key = 'p'}\\
-                1.5 \cdot 10^6 & \text{key = 'h'}
-                \end{cases}
         """
         if key == 'p':
-            return 0.5e5
+            fluid = single_fluid(c.fluid_data)
+            if fluid is not None:
+                return c.fluid.wrapper[fluid]._p_crit / 2
+            else:
+                return 1e5
         elif key == 'h':
-            return 1.5e6
+            fluid = single_fluid(c.fluid_data)
+            if fluid is not None:
+                if c.p.val_SI >= c.fluid.wrapper[fluid]._p_crit:
+                    temp = c.fluid.wrapper[fluid]._T_crit * 1.2
+                    return h_mix_pT(c.p.val_SI, temp, c.fluid_data)
+                else:
+                    return h_mix_pQ(c.p.val_SI, 1, c.fluid_data, c.mixing_rule)
+            else:
+                temp = 1000
+                return h_mix_pT(c.p.val_SI, temp, c.fluid_data, c.mixing_rule)
 
     @staticmethod
     def initialise_target(c, key):
@@ -486,39 +461,29 @@ class Turbine(Turbomachine):
         -------
         val : float
             Starting value for pressure/enthalpy in SI units.
-
-            .. math::
-
-                val = \begin{cases}
-                2.5 \cdot 10^6 & \text{key = 'p'}\\
-                2 \cdot 10^6 & \text{key = 'h'}
-                \end{cases}
         """
         if key == 'p':
-            return 2.5e6
+            fluid = single_fluid(c.fluid_data)
+            if fluid is not None:
+                return c.fluid.wrapper[fluid]._p_crit / 4 * 3
+            else:
+                return 10e5
         elif key == 'h':
-            return 2e6
+            fluid = single_fluid(c.fluid_data)
+            if fluid is not None:
+                if c.p.val_SI >= c.fluid.wrapper[fluid]._p_crit:
+                    temp = c.fluid.wrapper[fluid]._T_crit * 1.4
+                    return h_mix_pT(c.p.val_SI, temp, c.fluid_data)
+                else:
+                    return h_mix_pQ(c.p.val_SI, 1, c.fluid_data, c.mixing_rule) + 1e5
+            else:
+                temp = 500
+                return h_mix_pT(c.p.val_SI, temp, c.fluid_data, c.mixing_rule)
 
     def calc_parameters(self):
         r"""Postprocessing parameter calculation."""
         super().calc_parameters()
-
-        inl = self.inl[0]
-        outl = self.outl[0]
-        self.eta_s.val = (
-            (outl.h.val_SI - inl.h.val_SI)
-            / (
-                isentropic(
-                    inl.p.val_SI,
-                    inl.h.val_SI,
-                    outl.p.val_SI,
-                    inl.fluid_data,
-                    inl.mixing_rule,
-                    T0=inl.T.val_SI
-                )
-                - inl.h.val_SI
-            )
-        )
+        self.eta_s.val_SI = self.calc_eta_s()
 
     def exergy_balance(self, T0):
         r"""
