@@ -37,7 +37,9 @@ class TestClausiusRankine:
         self.Tamb = 20
         self.pamb = 1
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # create components
         splitter1 = Splitter('splitter 1')
@@ -57,7 +59,8 @@ class TestClausiusRankine:
         self.fwp_power = Bus('feed water pump power', P=0)
         self.fwp_power.add_comps(
             {'comp': fwp_turb, 'char': 1},
-            {'comp': fwp, 'char': 1, 'base': 'bus'})
+            {'comp': fwp, 'char': 1, 'base': 'bus'}
+        )
         # heat input bus
         self.heat = Bus('heat_input')
         self.heat.add_comps({'comp': steam_generator, 'base': 'bus'})
@@ -91,13 +94,14 @@ class TestClausiusRankine:
         # solve network
         self.nw.solve('design')
         self.nw.print_results()
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
 
     def test_exergy_analysis_perfect_cycle(self):
         """Test exergy analysis in the perfect clausius rankine cycle."""
         ean = ExergyAnalysis(
             self.nw, E_P=[self.power], E_F=[self.heat],
-            internal_busses=[self.fwp_power])
+            internal_busses=[self.fwp_power]
+        )
         ean.analyse(pamb=self.pamb, Tamb=self.Tamb)
         msg = (
             'Exergy destruction of this network must be 0 (smaller than ' +
@@ -153,7 +157,7 @@ class TestClausiusRankine:
 
         # solve network
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
         ean = ExergyAnalysis(
             self.nw, E_P=[self.power], E_F=[self.heat],
             internal_busses=[self.fwp_power])
@@ -196,7 +200,7 @@ class TestClausiusRankine:
             {'comp': self.nw.get_comp('pump'), 'char': 0.98, 'base': 'bus'})
         self.nw.add_busses(self.fwp_power)
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
         # miss out on internal bus in exergy_analysis
         ean = ExergyAnalysis(
             self.nw, E_P=[self.power], E_F=[self.heat])
@@ -222,7 +226,7 @@ class TestClausiusRankine:
         )
         self.nw.add_busses(self.fwp_power)
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
         # no exergy losses in this case
         ean = ExergyAnalysis(
             self.nw, E_P=[self.power], E_F=[self.heat],
@@ -275,7 +279,9 @@ class TestRefrigerator:
         self.Tamb = 20
         self.pamb = 1
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # create components
         va = Valve('expansion valve')
@@ -315,7 +321,7 @@ class TestRefrigerator:
 
         # solve network
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
 
     def test_exergy_analysis_bus_conversion(self):
         """Test exergy analysis at product exergy with T < Tamb."""
@@ -343,7 +349,9 @@ class TestCompressedAirIn:
 
         # compressor part
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # components
         amb = Source('air intake')
@@ -375,7 +383,7 @@ class TestCompressedAirIn:
 
         # solve network
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
 
     def test_exergy_analysis_bus_conversion(self):
         """Test exergy analysis at product exergy with T < Tamb."""
@@ -401,7 +409,9 @@ class TestCompressedAirOut:
 
         # turbine part
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # components
         cas = Source('compressed air storage')
@@ -439,7 +449,7 @@ class TestCompressedAirOut:
 
         # solve network
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
 
     def test_exergy_analysis_bus_conversion(self):
         """Test exergy analysis at product exergy with T < Tamb."""
@@ -466,7 +476,7 @@ class TestCompressedAirOut:
         c = self.nw.get_conn('outlet')
         c.set_attr(T=self.Tamb - 20)
         self.nw.solve('design')
-        self.nw._convergence_check()
+        self.nw.assert_convergence()
 
         ean.analyse(pamb=self.pamb, Tamb=self.Tamb)
 
@@ -490,7 +500,9 @@ class TestCompression:
 
         # turbine part
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # components
         so = Source('inlet')
@@ -569,7 +581,9 @@ class TestExpansion:
 
         # turbine part
         self.nw = Network()
-        self.nw.set_attr(p_unit='bar', T_unit='C', h_unit='kJ / kg')
+        self.nw.units.set_defaults(**{
+            "pressure": "bar", "temperature": "degC", "enthalpy": "kJ/kg"
+})
 
         # components
         so = Source('inlet')
